use common::PORT_NUM;
use std::error::Error;
use std::process::{Command, Stdio};
use std::net::{ToSocketAddrs, Ipv4Addr, SocketAddr, SocketAddrV4, TcpStream, TcpListener};
use std::io::{Read, Write};
use std::io::Result as IoResult;
use std::time::Duration;

// We're gonna try doing this with a struct that can "run itself" in a loop. This **should** make
// it easier to run in a thread: Just move the listener into a newly spawned thread
// When we start, Listener is just going to be a thin wrapper around TcpListener


pub struct Listener {
    tcl: TcpListener,
}

impl Listener {
    pub fn new() -> Listener {
        Listener {tcl: TcpListener::bind(format!("0.0.0.0:{?}", PORT_NUM))}
    }

    pub fn listen(&self) {
        for stream in self.tcl.incoming() {
            match stream {
                Ok(stream)  =>  self.in_process(stream)
                _           =>  {println!("Had a connection, lost it.")}
            }
        }
    }

    fn in_process(&self, stream: TcpStream) {
        let mut buf: [u8] = [0; 16]; // Tiny buffers for tiny messages
        let mut bufs = "";
        match stream.read_to_string(&bufs) {
            Ok(n)   => {},
            _       => {return}
        }

        match bufs {
            "hb"    =>  println!("Received heartbeat"),
            _       =>  println!(bufs)
        }
    }
}
